#pragma rtGlobals=2		// Use modern global access method.
#pragma ModuleName=GizmoUtil
#pragma IgorVersion = 6.20
#pragma version = 0.02
#include "ColorNames"

Static Constant GIZMO_MARKER_END_SIZE = 0.07		// puts boxes on ends of 3D marker (you can OverRide this in the Main procedure)
Static Constant GIZMO_MARKER_END_TYPE = 1			// 0=box on ends of lines, 1=sphere on ends of lines (you can OverRide this in the Main procedure)

//	MakeGizmocubeCorners(xyz)	Make a scatter wave that contains xyz and is CUBICAL
//	AddGizmoTitleGroup()			Add the title to a gizmo. Returns name of item to include in the display list
//	setGizmoAxisLabels()			set the three axis labels
//
//	isGizmoObjectDisplayed()		returns -1 if object no found on displayList, if found return place in displayList
//	GetGizmoObjects()				returns list of objects of type "type", from top gizmo or named gizmo
//	GizmoListScatterWaves()		return list of all scatter plots, except 'scatterMarker0'
//	GizmoListIsoSurfaceWaves()	return list of all iso surface waves

//Questionable:

//	AppendParametricXYZ2Gizmo()	Adds a parametric XYZ wave to the gizmo with its associated RGBA wave, returns name of object for parametricXYZ
//	GizmosWithWave()				Only used by AppendParametricXYZ2Gizmo(), Maybe this should not be here:




//  ============================================================================  //
//  =========================== Start of Add to orModify Gizmo ===========================  //

// Make a scatter wave that contains xyz and is CUBICAL
Function/WAVE MakeGizmocubeCorners(xyz)
	Wave xyz							// xyz[N][3], or xyz[nx][ny][nz]

	Variable Xlo=NaN, Xhi=NaN, Ylo=NaN, Yhi=NaN, Zlo=NaN, Zhi=NaN
	if (WaveDims(xyz)==3)				// a scaled 3D array
		Xlo = DimOffset(xyz,0)
		Xhi = DimDelta(xyz,0)*(DimSize(xyz,0)-1) + Xlo
		Ylo = DimOffset(xyz,1)
		Yhi = DimDelta(xyz,1)*(DimSize(xyz,1)-1) + Ylo
		Zlo = DimOffset(xyz,2)
		Zhi = DimDelta(xyz,2)*(DimSize(xyz,2)-1) + Zlo
		Variable swap
		if (Xlo>Xhi)
			swap = Xlo
			Xlo = Xhi
			Xhi = swap
		endif
		if (Ylo>Yhi)
			swap = Ylo
			Ylo = Yhi
			Yhi = swap
		endif
		if (Zlo>Zhi)
			swap = Zlo
			Zlo = Zhi
			Zhi = swap
		endif
	else
		Variable N=DimSize(xyz,0)		// a list of (xyz) triplets, xyz[N][3]
		Make/N=(N)/FREE/D cubeCorners_All
		cubeCorners_All = xyz[p][0]
		WaveStats/Q cubeCorners_All
		Xlo=V_min; Xhi=V_max
		cubeCorners_All = xyz[p][1]
		WaveStats/Q cubeCorners_All
		Ylo=V_min; Yhi=V_max
		cubeCorners_All = xyz[p][2]
		WaveStats/Q cubeCorners_All
		Zlo=V_min; Zhi=V_max
	endif

	Variable dX=Xhi-Xlo, dY=Yhi-Ylo, dZ=Zhi-Zlo
	Variable d = max(max(dX,dY),dZ)/2, mid

	Variable printIt = strlen(GetRTStackInfo(2))<=0
	if (printIt)
		printf "X = [%g, %g], 	=%g\r",Xlo,Xhi,dX
		printf "Y = [%g, %g], 		=%g\r",Ylo,Yhi,dY
		printf "Z = [%g, %g], 	=%g\r",Zlo,Zhi,dZ
	endif
	mid = (Xlo+Xhi)/2;		Xlo = mid-d;	Xhi = mid+d
	mid = (Ylo+Yhi)/2;		Ylo = mid-d;	Yhi = mid+d
	mid = (Zlo+zhi)/2;		Zlo = mid-d;	Zhi = mid+d
	if (printIt)
		print " "
		printf "X = [%g, %g], 	=%g\r",Xlo,Xhi,Xhi-Xlo
		printf "Y = [%g, %g], 		=%g\r",Ylo,Yhi,Yhi-Ylo
		printf "Z = [%g, %g], 	=%g\r",Zlo,Zhi,Zhi-Zlo
	endif

	String name=CleanupName(NameOfWave(xyz)+"Corners",0)
	Make/N=(8,3)/O $name=NaN
	Wave corners = $name
	corners[0][0] = Xlo;	corners[0][1] = Ylo; 	corners[0][2] = Zlo
	corners[1][0] = Xhi;	corners[1][1] = Ylo; 	corners[1][2] = Zlo
	corners[2][0] = Xlo;	corners[2][1] = Yhi; 	corners[2][2] = Zlo
	corners[3][0] = Xhi;	corners[3][1] = Yhi; 	corners[3][2] = Zlo
	corners[4][0] = Xlo;	corners[4][1] = Ylo; 	corners[4][2] = Zhi
	corners[5][0] = Xhi;	corners[5][1] = Ylo; 	corners[5][2] = Zhi
	corners[6][0] = Xlo;	corners[6][1] = Yhi; 	corners[6][2] = Zhi
	corners[7][0] = Xhi;	corners[7][1] = Yhi; 	corners[7][2] = Zhi
	String wnote="waveClass=GizmoCorners;"
	wnote = ReplaceStringByKey("sourceWave",wnote,NameOfWave(xyz),"=")
	Note/K corners, wnote
	return corners
End


// Add the title to a gizmo. Returns name of item to include in the display list
Function/T AddGizmoTitleGroup(groupName,title1,[title2,title3,title4,pos])
	String groupName			// probably "groupTitle", If this is empty, then a unique name will be assigned
	String title1
	String title2,title3,title4
	String pos
	title2 = SelectString(ParamIsDefault(title2),title2,"")
	title3 = SelectString(ParamIsDefault(title3),title3,"")
	title4 = SelectString(ParamIsDefault(title4),title4,"")
	pos = SelectString(ParamIsDefault(pos),pos,"TL")
	pos = SelectString(strlen(pos),"TL",pos)	// defaults to top left

	if (WhichListItem(pos,"TL;BL;")<0)
		return ""
	elseif (strlen(title1)<1)
		return ""
	elseif (CheckName(groupName,5))
		if (strlen(groupName)<1)						// create a unique group name
			NewDataFolder/O root:Packages				// ensure Packages exists
			NewDataFolder/O root:Packages:JZT_GizmoUtility	// ensure geometry exists
			if (exists("root:Packages:JZT_GizmoUtility:stringNumber")!=2)
				Variable/G root:Packages:JZT_GizmoUtility:stringNumber=-1
			endif
			NVAR stringNumber = root:Packages:JZT_GizmoUtility:stringNumber
			stringNumber += 1
			groupName = "gizmoStringGroup"+num2istr(stringNumber)
		endif
		groupName = CleanupName(groupName,0)
	endif

	Variable i
	for (i=0;i<3 && strlen(title2)==0;i+=1)	// don't leave empty strings between full ones
		title2 = title3							// e.g. "aa","","cc","dd"  --> "aa,"cc","dd",""
		title3 = title4
		title4 = ""
	endfor
	for (i=0;i<2 && strlen(title3)==0;i+=1)
		title3 = title4
		title4 = ""
	endfor

	// ************************* Group Object Start *******************
	Execute "AppendToGizmo group,name="+groupName
	Execute "ModifyGizmo currentGroupObject=\""+groupName+"\""
	Execute "AppendToGizmo string=\""+title1+"\",strFont=\"Geneva\",name=Title1"
	Execute "ModifyGizmo modifyObject=Title1 property={Clipped,0}"
	if (strlen(title2))
		Execute "AppendToGizmo string=\""+title2+"\",strFont=\"Geneva\",name=Title2"
		Execute "ModifyGizmo modifyObject=Title2 property={Clipped,0}"
	endif
	if (strlen(title3))
		Execute "AppendToGizmo string=\""+title3+"\",strFont=\"Geneva\",name=Title3"
		Execute "ModifyGizmo modifyObject=Title3 property={Clipped,0}"
	endif
	if (strlen(title4))
		Execute "AppendToGizmo string=\""+title4+"\",strFont=\"Geneva\",name=Title4"
		Execute "ModifyGizmo modifyObject=Title4 property={Clipped,0}"
	endif

	if (stringmatch(pos,"TL"))
		Execute "ModifyGizmo setDisplayList=0, opName=translateTitle, operation=translate, data={-1.9,1.9,0}"
	elseif (stringmatch(pos,"BL"))
		Execute "ModifyGizmo setDisplayList=0, opName=translateTitle, operation=translate, data={-1.9,-1.95,0}"
	endif
	Execute "ModifyGizmo setDisplayList=1, opName=rotateTitle, operation=rotate, data={180,1,0,0}"
	Execute "ModifyGizmo setDisplayList=2, opName=scaleTitle, operation=scale, data={0.1,0.1,0.1}"
	Execute "ModifyGizmo setDisplayList=3, object=Title1"
	if (strlen(title2))
		Execute "ModifyGizmo setDisplayList=4, opName=translateTitle2, operation=translate, data={0,1,0}"
		Execute "ModifyGizmo setDisplayList=5, opName=scaleTitle2, operation=scale, data={0.8,0.8,0.8}"
		Execute "ModifyGizmo setDisplayList=6, object=Title2"
		if (strlen(title3))
			Execute "ModifyGizmo setDisplayList=7, opName=translateTitle3, operation=translate, data={0,1,0}"
			Execute "ModifyGizmo setDisplayList=8, opName=scaleTitle3, operation=scale, data={0.7,0.7,0.7}"
			Execute "ModifyGizmo setDisplayList=9, object=Title3"
			if (strlen(title4))
				Execute "ModifyGizmo setDisplayList=10, opName=translateTitle4, operation=translate, data={0,1.5,0}"
				Execute "ModifyGizmo setDisplayList=11, object=Title4"
			endif
		endif
	endif
	Execute "ModifyGizmo currentGroupObject=\"::\""
	// ************************* Group Object End *******************
	return groupName
End


// Add a marker to a gizmo. Returns name of item to include in the display list
Function/T AddGizmoMarkerGroup(groupName,[rgba,alpha])
	String groupName			// probably "groupTitle", If this is empty, then a unique name will be assigned
	String rgba					// red, green, blue, or "" is black, or you can give your own rgba as "1,1,0,0.5"
	Variable alpha
	rgba = SelectString(ParamIsDefault(rgba),rgba,"")
	alpha = ParamIsDefault(alpha) ? 1 : alpha
	alpha = numtype(alpha) ? 1 : limit(alpha,0,1)


	if (CheckName(groupName,5))						// invalid groupName passed, create one
		if (strlen(groupName)<1)						// create a unique group name
			NewDataFolder/O root:Packages				// ensure Packages exists
			NewDataFolder/O root:Packages:JZT_GizmoUtility	// ensure geometry exists
			if (exists("root:Packages:JZT_GizmoUtility:stringNumber")!=2)
				Variable/G root:Packages:JZT_GizmoUtility:MarkerNumber=-1
			endif
			NVAR MarkerNumber = root:Packages:JZT_GizmoUtility:MarkerNumber
			MarkerNumber += 1
			groupName = "CrossPathGroup"+num2istr(MarkerNumber)
		endif
		groupName = CleanupName(groupName,0)
	endif
	if (CheckName(groupName,5))						// invalid groupName passed, create one
		return ""
	endif
	Variable r,g,b,a
	sscanf rgba,"%g,%g,%g,%g", r,g,b,a
	if (V_flag!=4)
		Wave rgb=color2RGB(rgba,1)					// a wide range of color names is available
		if (rgb[0]>0 && rgb[1]>0 && rgb[2]>0 && alpha>=0)
			sprintf rgba,"%g,%g,%g,%g", rgb[0],rgb[1],rgb[2],alpha
		else
			rgba = ""
		endif
	endif

	// ************************* Group Object Start *******************
	Execute "AppendToGizmo group,name="+groupName
	Execute "ModifyGizmo currentGroupObject=\""+groupName+"\""
	String str
	Variable v = GIZMO_MARKER_END_SIZE>0 ? 1-2*GIZMO_MARKER_END_SIZE : 1
	sprintf str, "AppendToGizmo line={%g,0,0,%g,0,0}, name=lineX",-v,v	;	Execute str
	sprintf str, "AppendToGizmo line={0,%g,0,0,%g,0}, name=lineY",-v,v	;	Execute str
	sprintf str, "AppendToGizmo line={0,0,%g,0,0,%g}, name=lineZ",-v,v	;	Execute str
	if (GIZMO_MARKER_END_SIZE>0)
		Variable endType=limit(GIZMO_MARKER_END_TYPE,0,1)
		if (endType==0)							// boxes
			sprintf str, "AppendToGizmo box={%g,%g,%g},name=endMarkers",GIZMO_MARKER_END_SIZE,GIZMO_MARKER_END_SIZE,GIZMO_MARKER_END_SIZE
			Execute str
			Execute "ModifyGizmo modifyObject=endMarkers property={calcNormals,1}"
		elseif (endType==1)						// spheres
			sprintf str, "AppendToGizmo sphere={%g,25,25},name=endMarkers",GIZMO_MARKER_END_SIZE
			Execute str
		endif
	endif
	Execute "AppendToGizmo attribute lineWidth=2, name=lineWidthCross"
	if (strlen(rgba))
		Execute "AppendToGizmo attribute color={"+rgba+"},name=colorLine"
	endif
	Execute "ModifyGizmo setDisplayList=0, opName=pushAttribute0, operation=pushAttribute, data=5"
	Execute "ModifyGizmo setDisplayList=1, attribute=lineWidthCross"
	if (strlen(rgba))
		Execute "ModifyGizmo setDisplayList=2, attribute=colorLine"
	endif
	Execute "ModifyGizmo setDisplayList=-1, object=lineX"
	Execute "ModifyGizmo setDisplayList=-1, object=lineY"
	Execute "ModifyGizmo setDisplayList=-1, object=lineZ"
	if (GIZMO_MARKER_END_SIZE>0)
		v = 1 - GIZMO_MARKER_END_SIZE
		sprintf str, "ModifyGizmo setDisplayList=-1, opName=translateXminus, operation=translate, data={%g,0,0}",-v	;	Execute str
		Execute "ModifyGizmo setDisplayList=-1, object=endMarkers"
		sprintf str, "ModifyGizmo setDisplayList=-1, opName=translateXplus, operation=translate, data={%g,0,0}",2*v	;	Execute str
		Execute "ModifyGizmo setDisplayList=-1, object=endMarkers"
		sprintf str, "ModifyGizmo setDisplayList=-1, opName=translateYminus, operation=translate, data={%g,%g,0}",-v,-v ; Execute str
		Execute "ModifyGizmo setDisplayList=-1, object=endMarkers"
		sprintf str, "ModifyGizmo setDisplayList=-1, opName=translateYplus, operation=translate, data={0,%g,0}",2*v	;	Execute str
		Execute "ModifyGizmo setDisplayList=-1, object=endMarkers"
		sprintf str, "ModifyGizmo setDisplayList=-1, opName=translateZminus, operation=translate, data={0,%g,%g}",-v,-v ; Execute str
		Execute "ModifyGizmo setDisplayList=-1, object=endMarkers"
		sprintf str, "ModifyGizmo setDisplayList=-1, opName=translateZplus, operation=translate, data={0,0,%g}",2*v	;	Execute str
		Execute "ModifyGizmo setDisplayList=-1, object=endMarkers"
	endif
	Execute "ModifyGizmo setDisplayList=-1, opName=popAttribute0, operation=popAttribute"
	Execute "ModifyGizmo currentGroupObject=\"::\""
	// ************************* Group Object End *******************

	return groupName
End


Function setGizmoAxisLabels(xlabel,ylabel,zlabel)
	String xlabel,ylabel,zlabel
	if (itemsInList(WinList("*",";","WIN:4096"))==0)
		return 1
	endif

	Execute "GetGizmo objectItemExists=axes0"
	NVAR V_Flag=V_Flag
	if (!V_flag)
		Execute "AppendToGizmo Axes=boxAxes,name=axes0"
	endif
	String cmd

	if (stringmatch(xlabel,"empty"))
		Execute "ModifyGizmo ModifyObject=axes0,property={0,axisLabel,0}"
	elseif (strlen(xlabel))
		Execute "ModifyGizmo ModifyObject=axes0,property={0,ticks,3}"
		Execute "ModifyGizmo ModifyObject=axes0,property={0,axisLabel,1}"
		sprintf cmd, "ModifyGizmo ModifyObject=axes0,property={0,axisLabelText,\"%s\"}",xlabel
		Execute cmd
		Execute "ModifyGizmo ModifyObject=axes0,property={0,axisLabelDistance,0.05}"
		Execute "ModifyGizmo ModifyObject=axes0,property={0,axisLabelScale,0.5}"
	endif

	if (stringmatch(ylabel,"empty"))
		Execute "ModifyGizmo ModifyObject=axes0,property={1,axisLabel,0}"
	elseif (strlen(ylabel))
		Execute "ModifyGizmo ModifyObject=axes0,property={1,ticks,3}"
		Execute "ModifyGizmo ModifyObject=axes0,property={1,axisLabel,1}"
		sprintf cmd, "ModifyGizmo ModifyObject=axes0,property={1,axisLabelText,\"%s\"}",ylabel
		Execute cmd
		Execute "ModifyGizmo ModifyObject=axes0,property={1,axisLabelDistance,0.05}"
		Execute "ModifyGizmo ModifyObject=axes0,property={1,axisLabelScale,0.5}"
	endif

	if (stringmatch(zlabel,"empty"))
		Execute "ModifyGizmo ModifyObject=axes0,property={2,axisLabel,0}"
	elseif (strlen(zlabel))
		Execute "ModifyGizmo ModifyObject=axes0,property={2,ticks,3}"
		Execute "ModifyGizmo ModifyObject=axes0,property={2,axisLabel,1}"
		sprintf cmd, "ModifyGizmo ModifyObject=axes0,property={2,axisLabelText,\"%s\"}",zlabel
		Execute cmd
		Execute "ModifyGizmo ModifyObject=axes0,property={2,axisLabelDistance,0.2}"
		Execute "ModifyGizmo ModifyObject=axes0,property={2,axisLabelScale,0.5}"
	endif
End

//  =========================== End of Add to orModify Gizmo ============================  //
//  ============================================================================  //



//  ============================================================================  //
//  ====================== Start of Examining things Displayed in a Gizmo =====================  //

Function isGizmoObjectDisplayed(object,[gizmo])		// returns -1 if object no found on displayList, if found return place in displayList
	String object						// name of a gizmo object
	String gizmo
	String Nswitch = ""
	if (!ParamIsDefault(gizmo) && strlen(gizmo))
		if (WinType(gizmo)!=13)
			return -1
		endif
		Nswitch = "/N="+gizmo
	endif

	Execute "GetGizmo/Z"+Nswitch+" displayList"
	String displayList=StrVarOrDefault("S_DisplayList","")
	KillWaves/Z TW_DisplayList
	KillStrings/Z S_DisplayList

	String item, find="*, object="+object
	Variable i,i0, num=-1
	for (i=0;i<ItemsInList(displayList);i+=1)
		item = StringFromList(i,displayList)
		if (stringmatch(item,find))
			i0 = strsearch(item," setDisplayList=",0,2)
			if (i0>0)
				i0 += strlen(" setDisplayList=")
				num = str2num(item[i0,Inf])
			endif
			break
		endif
	endfor
	num = num>=0 ? num : -1
	return num
End



Function/T GetGizmoObjects(type,[gizmo,exclude])		// returns list of objects of type "type", from top gizmo or named gizmo
	String type						// type of object to look for, e.g. "scatter"
	String gizmo
	String exclude					// an optional list of objects to exclude

	String Nswitch = ""
	if (!ParamIsDefault(gizmo) && strlen(gizmo))
		if (WinType(gizmo)!=13)
			return ""
		endif
		Nswitch = "/N="+gizmo
	endif
	if (ParamIsDefault(exclude))
		exclude = ""
	endif

	Execute "GetGizmo/Z"+Nswitch+" objectList"
	String objectList=StrVarOrDefault("S_gizmoObjectList","")
	KillWaves/Z TW_gizmoObjectList
	KillStrings/Z S_gizmoObjectList
	String item, objects="", objName
	Variable i
	for (i=0;i<ItemsInList(objectList,";");i+=1)
		item = StringFromList(i,objectList,";")
		if (strsearch(item,"AppendToGizmo "+type+"=",0,2)==1)
			objName = TrimFrontBackWhiteSpace(StringByKey("name",item,"=",","))
			if (WhichListItem(objName,exclude)<0)
				objects += objName+";"
			endif
		endif
	endfor
	return objects
End


Function/T GizmoListScatterWaves()				// get list of all scatter plots, except 'scatterMarker0'
	Execute "ModifyGizmo stopRotation"
	Execute "GetGizmo displayList"					// list of all displayed objects
	Execute "GetGizmo objectList"					// find name of wave with marker position
	Wave/T TW_DisplayList=TW_DisplayList, TW_gizmoObjectList=TW_gizmoObjectList

	String keyList=""
	String str, wname, list, scatterName
	Variable i,i0,i1
	for (i=0;i<numpnts(TW_gizmoObjectList);i+=1)
		str = TW_gizmoObjectList[i]

		i0 = strsearch(str,"AppendToGizmo ",0)
		if (i0<0 || i0>1)							// only process AppendToGizmo commands
			continue
		endif
		list = str[i0+strlen("AppendToGizmo "),Inf]

		scatterName = StringByKey("name",list,"=",",")
		if (stringmatch(scatterName,"scatterMarker0"))	// skip scatterMarker0
			continue
		endif

		wname = StringByKey("Scatter",list,"=",",")
		if (strlen(wname)<1)
			continue
		endif
		keyList = ReplaceStringByKey(wname,keyList,scatterName,"=")
	endfor
	KillWaves/Z TW_DisplayList, TW_gizmoObjectList
	KIllStrings/Z S_DisplayList, S_gizmoObjectList
	return keyList
End


Function/T GizmoListIsoSurfaceWaves()				// get list of all iso surface waves
	Execute "ModifyGizmo stopRotation"
	Execute "GetGizmo displayList"					// list of all displayed objects
	Execute "GetGizmo objectList"					// find name of wave with marker position
	Wave/T TW_DisplayList=TW_DisplayList, TW_gizmoObjectList=TW_gizmoObjectList

	String keyList=""
	String str, wname, list, scatterName
	Variable i,i0,i1
	for (i=0;i<numpnts(TW_gizmoObjectList);i+=1)
		str = TW_gizmoObjectList[i]

		i0 = strsearch(str,"AppendToGizmo ",0)
		if (i0<0 || i0>1)							// only process AppendToGizmo commands
			continue
		endif
		list = str[i0+strlen("AppendToGizmo "),Inf]

		scatterName = StringByKey("name",list,"=",",")
		wname = StringByKey("isoSurface",list,"=",",")
		if (strlen(wname)<1)
			continue
		endif
		keyList = ReplaceStringByKey(wname,keyList,scatterName,"=")
	endfor
	KillWaves/Z TW_DisplayList, TW_gizmoObjectList
	KIllStrings/Z S_DisplayList, S_gizmoObjectList
	return keyList
End

//  ====================== End of Examining things Displayed in a Gizmo ======================  //
//  ============================================================================  //



//  ============================================================================  //
//  ======================= Start of QUESTIONABLE items for this file =======================  //

// Will add a parametric XYZ wave to the gizmo with its associated RGBA wave, this is a pretty generic routine
// returns name of object for parametricXYZ
Static Function/T AppendParametricXYZ2Gizmo(parametricXYZ,class)
	Wave parametricXYZ
	String class							// wave class

	if(exists("NewGizmo")!=4)			// Do nothing if the Gizmo XOP is not available.
		DoAlert 0, "Gizmo XOP must be installed"
		return ""
	endif

	if (!WaveExists(parametricXYZ))
		class = SelectString(strlen(class),"*",class)
		String wStr="",wList = WaveListClass(class,"*","DIMS:3,MAXLAYERS:3,MINLAYERS:3")
		if (ItemsInList(wList)<1)
			DoAlert 0,"Parametric XYZ wave not found"
			return ""
		elseif (ItemsInList(wList)==1)
			wStr = StringFromLIst(0,wList)
		else
			Prompt wStr, "Parametric XYZ wave",popup, wList
			DoPrompt "Parametric XYZ ("+class+")",wStr
			if (V_flag)
				return ""
			endif
		endif
		Wave parametricXYZ = $wStr
		printf "AppendParametricXYZ2Gizmo(%s)\r",NameOfWave(parametricXYZ)
	endif
	if (!WaveExists(parametricXYZ))
		DoAlert 0,"Parametric XYZ wave not found"
		return ""
	endif
	if (WaveDims(parametricXYZ)!=3 || DimSize(parametricXYZ,2)!=3)
		DoAlert 0,"Parametric XYZ wave "+NameOfWave(parametricXYZ)+" is not of dimension [N][M][3]"
		return ""
	endif

	Variable i,j
	String win=GizmosWithWave(parametricXYZ)
	String xyzName = GetWavesDataFolder(parametricXYZ,2)
	if (strlen(win)>0)
		DoWindow/F $win				// for wave already in gizmo, bring it to front
		Execute "GetGizmo/N="+win+"/Z objectList"
		Wave/T TW_gizmoObjectList=TW_gizmoObjectList
		for (i=0,j=-1; i<numpnts(TW_gizmoObjectList) && j<0; i+=1)
			j = strsearch(TW_gizmoObjectList[i],xyzName,0,2)
		endfor
		return StringByKey("name",TW_gizmoObjectList[i-1],"=",",")
	endif

	String wnote=note(parametricXYZ)
	Wave sourceWave = $StringByKey("source", wnote,"=")
	if (!WaveExists(sourceWave))
		return ""
	endif
	win = GizmosWithWave(sourceWave)
	if (strlen(win)<1)
		return ""
	endif
	DoWindow/F $win					// bring gizmo to front, and then append the surface
	DoUpdate

	// Find unsued name for surface, starting with "ParametricSurf0"
	Execute "GetGizmo/N="+win+"/Z objectNameList"
	SVAR S_ObjectNames=S_ObjectNames
	String rgbaName=StringByKey("RGBA",wnote,"="), surfaceName
	for (i=0,j=0; j>=0; i+=1)
		surfaceName = "ParametricSurf"+num2istr(i)
		j = WhichListItem(surfaceName, S_ObjectNames)
	endfor
	Execute "AppendToGizmo Surface="+xyzName+",name="+surfaceName
	Execute "ModifyGizmo ModifyObject="+surfaceName+" property={ srcMode,4}"
	Execute "ModifyGizmo modifyObject="+surfaceName+" property={calcNormals,1}"
	if (exists(rgbaName)==1)
		Execute "ModifyGizmo ModifyObject="+surfaceName+" property={ surfaceColorType,3}"
		Execute "ModifyGizmo ModifyObject="+surfaceName+" property={ surfaceColorWave,"+rgbaName+"}"
	endif
	Execute "ModifyGizmo setDisplayList=-1, object="+surfaceName
	return surfaceName
End



// Maybe this should not be here:
Static Function/T GizmosWithWave(scatter)
	Wave scatter

	if (!WaveExists(scatter))
		return ""
	endif
	String list = WinList("*", ";", "WIN:4096" )	// list of possible gizmos
	Variable N=ItemsInlist(list)
	String str, wName=GetWavesDataFolder(scatter,2)
	Variable i
	for (i=N-1;i>=0;i-=1)
		str = WinRecreation(StringFromList(i,list), 0)
		if (strsearch(str,wName,0)<0)
			list = RemoveListItem(i,list)
		endif
	endfor
	N=ItemsInlist(list)

	if (N<1)
		return ""
	elseif (N==1)
		return StringFromList(0,list)
	endif

	String wGizmo
	Prompt wGizmo,"Which Gizmo?",popup,list
	DoPrompt "Gizmo?",wGizmo
	if (V_flag)
		return ""
	endif
	return wGizmo
End

//  ======================= End of QUESTIONABLE items for this file ========================  //
//  ============================================================================  //
