#pragma rtGlobals=1		// Use modern global access method.
#pragma IgorVersion = 4.0
#pragma version = 3.0


Function CallShowSpotSize()
	Variable sigma			// sigma for source (m)
	Variable sigmap			// sigmap for source (rad)
	Variable s1=64.5		// distance source to mirror (m)
	Variable s2=0.13		// distance mirror to focus (m)
	Variable tth				// 2-theta for middle of mirror (rad)
	Variable N=50000		// number of rays to trace

	String mirrorName=""	// mirror height
	String slopeName		// mirror slopes
	String slopeXname		// name of x-value for slopes (if needed)
	String resultName		// name for result wave
	String slopeNameChoices=WaveList("*Rad",";","")+WaveList("*slope*",";","")+WaveList("*deriv",";","")+WaveList("*angle*",";","")
	if (strlen(StringFromList(0, slopeNameChoices)))
		slopeName = StringFromList(0, slopeNameChoices)
		slopeXname = StringByKey("Xdata", note($slopeName),"=" )
	endif
	Prompt slopeName, "scaled mirror slope", popup, slopeNameChoices
	Prompt mirrorName, "scaled mirror height", popup, "_calculate_;"+WaveList("*height*",";","")
	Prompt slopeXname, "X wave for slope", popup, "_calculated_;"+WaveList("*X",";","")
	String s1list=MakePromptString(s1,"64.5 (source on 34);36.5 (L5 on 34);other") ; s1=0
	String s2list=MakePromptString(s2,"0.13 (long mirror);0.060 (short mirror);other") ; s2=0
	Prompt s1, "distance from mirror to source (m)", popup, s1list
	Prompt s2, "distance from mirror to focus (m)", popup, s2list
	NVAR debugLevel_G = root:Packages:KBmirror:debugLevel
	Variable debugLevel=debugLevel_G
	if (debugLevel)
		Variable/G x_or_angle_G
		Variable x_or_angle = x_or_angle_G
		Prompt x_or_angle, "scan x or angle", popup, "x;angle"
		Prompt debugLevel, "debugging level", popup, "0;1;2;3;4;5;6;7;8;9;10"
		debugLevel += 1					// popup menus start at 1 not 0
		DoPrompt "pick waves to use", s1,slopeName,slopeXname,mirrorName,debugLevel,x_or_angle
		s1 = str2num(StringFromList(s1-1,s1list))
		s2 = str2num(StringFromList(s2-1,s2list))
		debugLevel -= 1
		debugLevel_G = debugLevel
		x_or_angle_G = x_or_angle
		N = 101
		SetRandomSeed 0.5
	else
		Prompt N, "number of rays to trace points"
		DoPrompt "pick waves to use", s1,s2,slopeName,slopeXname,mirrorName, N
		if (V_flag)
			return 1
		endif
		s1 = str2num(StringFromList(s1-1,s1list))
		s2 = str2num(StringFromList(s2-1,s2list))
		if (numtype(s1) || numtype(s2))
			Prompt s1, "distance from mirror to source (m)"
			Prompt s2, "distance from mirror to focus (m)"
			s1 = str2num(StringFromList(0,s1list))		// reset to original
			s2 = str2num(StringFromList(0,s2list))
			DoPrompt "distances to use", s1,s2				// let user enter anything
			if (V_flag)
				return 1
			endif
		endif
	endif
	if (V_flag)
		return 1
	endif
	if (numtype(s1) || numtype(s2) || s1<1 || s2<.001)
		DoAlert 0, "cannot use s1="+num2str(s1)+"  or  s2="+num2str(s2)
		return 1
	endif
	Variable length = NumberByKey("length",note($slopeName),"=")	// used to set defaults
	String mm = num2istr(round(abs(length)*1000))
	resultName = "spots"+mm+"mm"

	tth=.006			// 2-theta for middle of mirror (rad)
	// note FWHM = sigma * 2 * sqrt(2*ln(2))
	if (stripeType(length)==1)
		sigma = 21e-6			// 21 m
		sigmap = 6.9e-6		// 6.9 rad
	elseif (stripeType(length)==2)
		sigma = 359e-6		// 359 m
		sigmap = 24e-6		// 24 rad
		if (s1<=10)			// at source of 10 m, assume small L5 slit
			sigma = 36e-6
		endif
	else
		Variable fl=NumberByKey("fl", note($slopeName),"=")
		s2 = (numtype(fl)) ? s2 : 1/(1/fl - 1/s1)
		s2 = (s2<.001 || numtype(s2)) ? 1.0 : s2
		String Ssigma, Ssigmap
		Prompt s2, "distance from center of mirror to focus (m)"
		Prompt Ssigma, "sigma", popup, "359 m (horiz);36 m (horiz@L5);21 m (vertical)"
		Prompt Ssigmap, "sigma'", popup, "24 rad (horiz);6.9 rad (vertical)"
		Prompt tth, "twice the grazing angle at mirror center (rad)"
		if (s1==10)
			Ssigma="36 m (horiz@L5)"
		endif
		DoPrompt "focus parameters", s2, tth, Ssigma, Ssigmap
		if (V_flag)
			return 1
		endif
		sigma = str2num(Ssigma)*1e-6
		sigmap = str2num(Ssigmap)*1e-6
	endif
	printf "CallShowSpotSize(s1=%g m, s2=%g m, N=%d, slope='%s'",s1,s2,N,slopeName
	if (cmpstr(slopeXname,"_calculated_"))
		printf "(%s)", slopeXname
	endif
	if (cmpstr(mirrorName,"_calculate_"))
		printf ", height='%s'", mirrorName
	endif
	if (debugLevel)
		if (x_or_angle==1)
			printf ", debug(x)=%d",debugLevel
		else
			printf ", debug(angle)=%d",debugLevel
		endif
	endif
	printf ")\r"

	// create needed slope and height waves
	if (!stringmatch(slopeXname, "_calculated_" ))			// convert to x-scaled wave
		Duplicate/O $slopeName, slope_temp__
		Variable Yscaling = AngleScalng2radians(WaveUnits($slopeName,-1))
		Variable Xscaling = LengthScalng2meters(WaveUnits($slopeXname,-1))
		Variable xlo,xhi
		Wave slopeX=$slopeXname
		xlo = floor(slopeX[0]*Xscaling*1000)		// lo limit in mm
		xhi = ceil(slopeX[inf]*Xscaling*1000)		// hi limit in mm
		String command
		sprintf command, "Interpolate/T=2/N=%d/E=1/Y=slope_temp__ %s /X=%s", 4*(xhi-xlo+1),slopeName, slopeXname
		Execute command							// make scaled slope wave
		slopeName = "slope_temp__"				// name of scaled slope to use from here on
		slope_temp__ *= Yscaling
		SetScale d 0,0,"rad", slope_temp__
		SetScale/P x DimOffset(slope_temp__,0)*Xscaling,DimDelta(slope_temp__,0)*Xscaling,"m", slope_temp__
	endif
	String killMe=""
	if (stringmatch(mirrorName, "_calculate_" ))	// get height from the slopes
		mirrorName = HeightFromScaledSlope($slopeName)
		killMe=mirrorName
	endif

	Variable goodRays				// number of rays that made it to the focal plane
	Wave mirror = $mirrorName	// scaled mirror height (m)
	Wave slope = $slopeName		// scaled slopes for the mirror (rad)
	Variable y0 = mirror(0)
	mirror -= y0					// ensure mirror(0) == 0
	Variable slope0 = slope(0)
	slope -= slope0					// ensure slope(0) == 0
	goodRays = ShowSpotSize(N,sigma,sigmap,s1,s2,tth,$mirrorName,$slopeName,resultName,debugLevel)

	if (debugLevel==0)
		KillWaves/Z slope_temp__
		if (strlen(killMe)>0)
			KillWaves/Z killMe
		endif
	endif

	printf "using '%s'   and   '%s':\r",mirrorName, slopeName
	if (N-goodRays>0)
		printf "there were %d points that missed the mirror out of %d\r", N-goodRays,N
	endif
	printf "FWHM = %g m   ",FWHM_of_Wave($resultName)*1e6
	printf "(s1=%g m,   s2=%g mm,   theta=%g mrad,   ",s1,s2*1000,tth/2*1000
	printf "sigma=%g m,   sigmap=%g rad)\r", sigma*1e6, sigmap*1e6
	printf "   best  spot size at this demag is %g m\r", sigma*1e6 * 2*sqrt(2*ln(2)) * s2/s1
End

Function ShowSpotSize(N,sigma,sigmap,s1,s2,tth,mirror,slope,resultName,debugLevel)
	Variable N			// number of rays to trace
	Variable sigma		// sigma for source (m)
	Variable sigmap		// sigmap for source (rad)
	Variable s1			// distance source to mirror (m)
	Variable s2			// distance mirror to focus (m)
	Variable tth			// 2-theta for middle of mirror (rad)
	Wave mirror		// scaled mirror height (m)
	Wave slope			// scaled slopes for the mirror (rad)
	String resultName	// name for result wave
	Variable debugLevel	// flag for debugging

	String yName = resultName+"_y"		// wave with y value for each ray traced
	String ypName = resultName+"_yp"		// y' value that goes with yName
	Make/N=(N)/O/D $yName, $ypName
	Wave ywave=$yName
	Wave ypwave=$ypName
	ywave = 0
	if (debugLevel)
		Variable width = DimDelta(mirror,0)*(numpnts(mirror)-1)
		Variable delta = DimOffset(mirror,0)
		SetScale/I x delta,delta+width,"m", $yName
	else
		SetScale/P x 0,1,"", $yName
	endif
	CopyScales $yName, $ypName
	SetScale d 0,0,"m", $yName
	SetScale d 0,0,"rad", $ypName
	Variable goodRays		// number of rays that made it to the focal plane
	goodRays = TraceManyRays(sigma,sigmap,s1,s2,tth,mirror,slope,ywave,ypwave,debugLevel)

	Make/N=101/O/D  $resultName
	Wave result = $resultName
	WaveStats/Q ywave
	SetScale/I x V_min,V_max,"m", result
	Histogram/B=2 ywave,$resultName
//	if (debugLevel==0)
//		KillWaves/Z spot_temp__
//	endif
	WaveStats/Q result
	result *= goodRays/N/V_max		// normalize to peak of 1 for good rays

	PauseUpdate; Silent 1
	Display /W=(5,42,435,326) $resultName
	ModifyGraph/Z gfSize=18, lsize=2, tick=2, minor=1, standoff=0, lowTrip=0.1, mirror=1
	ModifyGraph axOffset(left)=-2.8,axOffset(bottom)=-1.28
	String str
	sprintf str, "\\Z14   \\f02rays\\f00\r%d missed\r%d OK\r%d total", N-goodRays, goodRays, N
	TextBox/N=text0/F=0/S=3/X=3.33/Y=9.60 str
	sprintf str, "\\Z14fwhm=%.2f m", FWHM_of_Wave($resultName)*1e6
	TextBox/N=text1/F=0/S=3/A=LT/X=5/Y=11/B=1 str
	sprintf str, "\\Z10s1 = %g m", s1
	AppendText/N=text1 str
	sprintf str, "s2 = %g mm", s2*1000
	AppendText/N=text1 str
	sprintf str, "fl = %g mm", 1/(1/s1 + 1/s2)*1000
	AppendText/N=text1 str
	sprintf str, "\\F'Symbol'q\\F]0 = %g mrad", tth*1000/2
	AppendText/N=text1 str
	sprintf str, "\\F'Symbol's\\F]0 = %g m", sigma*1e6
	AppendText/N=text1 str
	sprintf str, "\\F'Symbol's\\F]0' = %g rad", sigmap*1e6
	AppendText/N=text1 str
	Cursor/P A $resultName 0
	return goodRays
End

Function MakeRaysPlot()
	Variable N=100				// number of lines to draw
	Variable xRange=.0005		// x range of plot (m)
	Prompt N, "number of rays to plot"
	Prompt xRange, "range of x to show (m)"
	String yName=""
	Prompt yName, "wave of y values for each ray", popup, WaveList("*_y",";","")

	DoPrompt "Choose rays to plot", N,xRange,yName
	if (V_flag)
		return 1
	endif
	Wave yWave=$yName
	Wave ypWave=$yName+"p"
	printf "MakeRaysPlot(N=%d, xRange=%g m, y[i]='%s')\r",N,xRange,yName
	if (exists(NameOfWave(ypWave))!=1)
		Abort "y' wave does not exits, should be '"+yName+"p'"
	endif

	Make/N=2/O rays__
	WaveStats/Q yWave
	rays__ = {V_min,V_max}
	Variable xlo=-xRange, xhi=xRange
	SetScale/I x xlo,xhi,"m", rays__
	SetScale d 0,0,"m", rays__
	PauseUpdate
	Display /W=(5,42,498,383) rays__
	ModifyGraph gfSize=18, mode=2, tick=2, mirror=1, minor=1, standoff=0
	ModifyGraph axOffset(left)=-3.4,axOffset(bottom)=-0.277778
	SetDrawLayer UserFront
	Variable i=0, ylo, yhi, ymax=V_max,ymin=V_min
	do
		if (!numtype(yWave[i]))
			ylo = ypWave[i]*xlo + yWave[i]
			yhi = ypWave[i]*xhi + yWave[i]
			SetDrawEnv xcoord= bottom,ycoord= left
			DrawLine xlo,ylo,xhi,yhi
			ymin = min( ymin,ylo)
			ymin = min( ymin,yhi)
			ymax = max(ymax,ylo)	
			ymax = max(ymax,yhi)
		endif
		i += 1
	while (i<N)
	SetAxis left ymin, ymax
	Cursor/P/F/H=1 A rays__ 0.5,0.75;Cursor/P/F/H=1 B rays__ 0.5,0.25
	ShowInfo
End

Function TraceManyRays(sigma,sigmap,s1,s2,tth,mirror,slope,spotList,spotListp,debugLevel)
	Variable sigma		// sigma for source (m)
	Variable sigmap		// sigmap for source (rad)
	Variable s1			// distance source to mirror (m)
	Variable s2			// distance mirror to focus (m)
	Variable tth			// 2-theta for middle of mirror (rad)
	Wave mirror		// scaled mirror height (m)
	Wave slope			// scaled slopes for the mirror (rad)
	Wave spotList		// holds y positions of points in focus (m)
	Wave spotListp		// holds y' of points in focus (rad)
	Variable debugLevel	// used for debugging, 0--> none

	Variable N=numpnts(spotList)	// number of rays to trace
	Variable tolerance = 1e-9		// used to find where ray hits mirror (dist along mirror, x)
	Variable debugTest=0

	// make the source
	Trace_MakeSource(sigma,sigmap,spotList,spotListp,debugLevel)
	debugTest += 1
	if (debugLevel==debugTest)
		WaveStats/Q spotList
		return V_npnts
	endif

	//	tranfer to position of mirror
	Trace_Translate(s1,spotList,spotListp)
	debugTest += 1
	if (debugLevel==debugTest)
		printf "returning after Trace_Translate A, debug=%d\r",debugTest
		WaveStats/Q spotList
		return V_npnts
	endif

	//	rotate to match mirror
	Trace_Rotate(tth/2,spotList,spotListp)
	debugTest += 1
	if (debugLevel==debugTest)
		printf "returning after Trace_Rotate A, debug=%d\r",debugTest
		WaveStats/Q spotList
		return V_npnts
	endif

	//	reflect from the mirror
	Trace_ReflectFromMirror(mirror,slope,tolerance,spotList,spotListp)
	debugTest += 1
	if (debugLevel==debugTest)
		printf "returning after Trace_ReflectFromMirror, debug=%d\r",debugTest
		WaveStats/Q spotList
		return V_npnts
	endif

	//	rotate again to line up with rays from mirror
	Trace_Rotate(tth/2,spotList,spotListp)
	debugTest += 1
	if (debugLevel==debugTest)
		printf "returning after Trace_Rotate B, debug=%d\r",debugTest
		WaveStats/Q spotList
		return V_npnts
	endif

	//	transfer to focus
	Trace_Translate(s2,spotList,spotListp)
	if (debugLevel)
		printf "returning after Trace_Translate B (last one), debug=%d\r",debugTest
		WaveStats/Q spotList
		return V_npnts
	endif

	WaveStats/Q spotList
	return V_npnts									// number of points that made it
End

Function Trace_MakeSource(sigma,sigmap,ywave,ypwave,debugLevel)
	// fill y,y' with rays (at position of source) and angle is relative to source (not mirror)
	Variable sigma		// sigma for source (m)
	Variable sigmap		// sigmap for source (rad)
	Wave ywave			// list of y	for each ray
	Wave ypwave		// list of y' (slopes, not angles)
	Variable debugLevel	// used for debugging, 0--> none

	Variable N=numpnts(ywave)	// number of rays to trace
	if (debugLevel)
		Variable/G x_or_angle_G
		Variable tth=.006, s1=36.5
		if (x_or_angle_G==1)
			ywave = x*tth/2 * 1.14			// 14% over
			ypwave = 0
		else
			ywave = 0
			ypwave = x*tth/2 * 1.14/s1
		endif
	else
		ywave = gnoise(sigma)					// position from source center
		ypwave = gnoise(sigmap)				// slope from source
	endif
End

Function Trace_Translate(dist,ywave,ypwave)
	// transfer rays a distance dist to the right
	// use positive distance to move to the right (putting source at left), so always use positive
	Variable dist		// distance to translate (m)
	Wave ywave			// list of y	for each ray
	Wave ypwave		// list of y' (slopes, not angles)
	ywave += ypwave * dist				// y-coord at new x=0
	// ypwave is unchanged by a translation
End

Function Trace_Rotate(angle,ywave,ypwave)
	// rotate rays an angle clockwise (rotate the frame counter-clockwise)
	// use positive angle going from source to mirror, AND from mirror to focus
	Variable angle		// angle to rotate (rad), +angle makes slopes more positive
	Wave ywave			// list of y	for each ray
	Wave ypwave		// list of y' (slopes, not angles)

	Variable cosa = cos(angle)
	Variable sina = sin(angle)
	ypwave = tan(atan(ypwave[p])-angle)	// new slope rotated clockwise by angle
	ywave = ywave[p] * (cosa - ypwave[p]*sina)
End

Function Trace_ReflectFromMirror(mirror,slope,tolerance,ywave,ypwave)
	Wave mirror		// scaled mirror height (m)
	Wave slope			// scaled slopes for the mirror (rad)
	Variable tolerance	// error in x of the returned intercept
	Wave ywave			// list of y	for each ray
	Wave ypwave		// list of y' (slopes, not angles)

	Variable N=numpnts(ywave)
	Variable xloStart,xhiStart		// range in x of the mirror (cannot go outside this)
	xloStart = DimOffset(mirror, 0)
	xhiStart = xloStart + DimDelta(mirror, 0)*(numpnts(mirror)-1)

	Variable xlo,ylo, xhi, yhi		// range in (x,y) of current search interval
	Variable ys						// position of ray at the source, always assume source at x=0
	Variable yp						// slope of ray at the source, (x=0 is the center of the mirror)
	Variable xm,ym					// new guesses at middle of lo and hi (position on mirror)
	Variable ymirror				// actual y position of mirror corresponding to xm
	Variable dx						// current step size in search for intersction
	Variable i						// index for search for intersection

	Variable j
	for(j=0;j<N;j+=1)				// do once for each ray
		ys = ywave[j]
		yp = ypwave[j]

		if (numtype(ywave[j]))					// do not trace rays that have already not made it
			continue
		endif

		xlo = xloStart
		xhi = xhiStart
		ylo = xlo*yp + ys							// check that ray hits mirror
		yhi = xhi*yp + ys
		if (ylo<mirror(xlo) || yhi > mirror(xhi))	// ray does not hit mirror
			ywave[j] = NaN
			ypwave[j] = NaN
			continue
		endif

		// ray hits mirror, find intersection of ray with mirror
		dx=xhi-xlo
		i = 0
		do
			xm = (xlo + xhi)/2
			ym = xm*yp + ys
			ymirror = mirror(xm)

			if (ym>ymirror)						// pick range with root
				xlo = xm ; ylo = ym
				dx = xhi-xm
			else
				xhi = xm ; yhi = ym
				dx = xm - xlo
			endif
			i += 1
		while (i<51 && abs(dx)>tolerance)			// max number of iterations
		if (abs(dx)>tolerance)
			DoAlert 1, "Did 50 iterations with error = "+num2str(ym-ymirror)+"\rcontinue?"
			if (V_flag!=1)
				Abort "aborting"
				return 1							// aborted
			endif
		endif

		// (xm, mirror(xm)) are now point where ray hits mirror in mirror's ref frame
		ypwave[j] = tan(atan(-yp) + 2*atan(slope(xm)))	// slope of exiting ray
		ywave[j] = mirror(xm) - ypwave[j] * xm			// position of exiting ray at x=0
	endfor
	return 0										// flags no error, processed OK
End

Function Graph_debugging()
	String yName=""
	Prompt yName, "wave of y values for each ray", popup, WaveList("*_y",";","")

	DoPrompt "Choose rays to plot", yName
	if (V_flag)
		return 1
	endif
	Wave yWave=$yName
	Wave ypWave=$yName+"p"
	if (exists(NameOfWave(ypWave))!=1)
		Abort "y' wave does not exits, should be '"+yName+"p'"
	endif
	Display /W=(263,47,788,437) yWave
	AppendToGraph/R ypWave
	ModifyGraph gfSize=18, lSize=2
	ModifyGraph rgb($yName+"p")=(0,0,65280)
	ModifyGraph offset($yName+"p")={0.00026178,-8.94507e-08}
	ModifyGraph tick=2, mirror(bottom)=1, minor=1, standoff=0
	ModifyGraph axOffset(left)=-2.2,axOffset(right)=-0.7
	Legend/N=text0/J/S=3/A=LT/X=6.91/Y=7.10 "\\s("+yName+") position \\Z12("+yName+")\\M"
	AppendText "\\s("+yName+"p) slope \\Z12("+yName+"p)\\M"
	Cursor/P A $yName 10;Cursor/P B $yName 99
	ShowInfo
End




































Function FWHM_of_Wave(spot)			// return fwhm of the spot profile
	Wave spot							// wave containing spot position, scaled

	Variable xlo, xhi, deltaInt
	WaveStats/Q spot
	deltaInt = V_max - V_min 
	FindLevel/Q/R=(-inf,V_maxloc)  spot, deltaInt*.5 + V_min
	xlo = V_LevelX
	FindLevel/Q/R=(V_maxloc,inf)  spot, deltaInt*.5 + V_min
	xhi = V_LevelX
	return (xhi-xlo)
End



